<?php
defined('ABSPATH') or die("you do not have access to this page!");

if (!function_exists('rsssl_send_php_security_headers')) {

	/**
	 * Enables the HTTP Strict Transport Security (HSTS) header also on non apache servers.
	 *
	 * @since 1.0.25
	 **/

	function rsssl_send_php_security_headers() {
		$wp_hsts = get_option("rsssl_hsts_no_apache");
		if ($wp_hsts && is_ssl()) {
			$preload = get_option("rsssl_hsts_preload");
			if (!$preload) {
				header( 'Strict-Transport-Security: max-age=31536000' );
			} else {
				header( 'Strict-Transport-Security: max-age=63072000; includeSubDomains; preload;' );
			}
		}
	}
	add_action( 'send_headers', 'rsssl_send_php_security_headers' );
}


if ( !function_exists('rsssl_secure_logged_in_cookie')) {

	/**
	 * Set a secure cookie, but only if the site is enabled for SSL, not per page.
	 * This setting can be used on multisite as well, as it will decide per site what setting to use.
	 * @since 2.0.10
	 * */

	function rsssl_secure_logged_in_cookie($secure_logged_in_cookie, $user_id, $secure){
		$options = get_option("rsssl_options");
		$ssl_enabled = isset($options['ssl_enabled']) ? $options['ssl_enabled'] : false;

		if (!defined('rsssl_pp_version') && $ssl_enabled) {
			return true;
		}
		return $secure_logged_in_cookie;
	}
	add_filter( 'secure_logged_in_cookie', 'rsssl_secure_logged_in_cookie' , 10, 3);
}

if ( !function_exists('rsssl_uses_default_folder_name')) {

	/**
	 *
	 * Check the current free plugin folder path and compare it to default path to detect if the plugin folder has been renamed
	 *
	 * @return boolean
	 *
	 * @since 3.1
	 *
	 */

	function rsssl_uses_default_folder_name() {
		$default_plugin_path = RSSSL()->really_simple_ssl->plugin_dir;
		$current_plugin_path
		                     = RSSSL()->really_simple_ssl->get_current_rsssl_free_dirname();
		if ( $default_plugin_path === $current_plugin_path ) {
			return true;
		} else {
			return false;
		}
	}
}

if ( !function_exists('is_rsssl_plugin_active')) {
	/**
	 *
	 * Check the current free plugin is active
	 *
	 * @since 3.1
	 *
	 * @return boolean
	 *
	 */

	function is_rsssl_plugin_active()
	{
		if (defined('rsssl_plugin')) {
			return true;
		} else {
			return false;
		}
	}
}